function [signal_clean, signal_noisy, parameterization_coeff, sigma, handle_signal, handle_coeff] = lib_generate_signal(N, degree, njumps, SNR)

% Function that randomly generates a piecewise polynomial signal of
% selected degree, number of segments, lenght and SNR
%
% INPUT: 
% N - length of signal
% njumps - number of jumps in parameterization coefficients
% degree - polynom degree
% SNR - signal to noise ration
%
% OUTPUT:
% signal clean - randomly generated signal
% signal_ noisy - noisy signal (signal_clean + noise)
% parameterization_coeff - parameterization coefficients of clean signal
% sigma
% handle_signal - number of figure of signal
% handle_coeff - number of figuer of coefficients

if nargin < 4
  SNR = 25
end 

if nargin < 3
  njumps = 4
end 

parameterization_coeff=zeros(N,degree);
matrix_of_d=ones(N,degree);
d = ((1:N)/N)';  %linearly growing

ind = ceil(N * rand(njumps,1));  %random sparse positions

for i=1:degree
    x=parameterization_coeff(:,i); % vector of k-th parametrization coefficients
    x(1) = randn(1,1);            %random value - start of the signal
    x(ind) = randn(njumps,1);        %random values
    x = cumsum(x);                %make a stepwise function
    parameterization_coeff(:,i)=x;
    matrix_of_d(:,i)=d.^(i-1);
   
end

signal_clean = sum(parameterization_coeff.*matrix_of_d,2);  %generate signal

%% add noise
ex = sum(signal_clean.^2);
sigma = sqrt( ex / ( N * 10 ^ ( SNR/10 ) ) );    %std. deviation of the noise
signal_noisy = signal_clean + sigma * randn(N,1);  %add noise

%% display the signals and parameters
handle_signal = figure('Name','Signal','NumberTitle','off');
plot(1:N,signal_clean);
hold on;
plot(1:N,signal_noisy,'r');
legend('clean', 'observed');

handle_coeff = figure('Name','Parameterization coeff','NumberTitle','off');
types='xo*xo*xo*';
colors='bbbcccyyy';

for i=1:degree
    plot(1:N,parameterization_coeff(:,i),[colors(i),types(i)])
    hold on
end



